#!/usr/bin/perl
#------------------------------------------------------------------------------
# Program   : s p i p . p l
#------------------------------------------------------------------------------
# Author    : Christian Dhl
# Created   : 23. Februar 2005
# Changed   : 28. Februar 2005
# Task      : S P I P - Simple Presentations in Perl
# Parameter : None, you have to call it from your presentation directory,
#             all informations are read from the file config.spip in the
#             current working directory.
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
# packages / pragmas:
#------------------------------------------------------------------------------
use strict;
use warnings;

use IO::File;
use XML::Simple;
#use Data::Dumper; $Data::Dumper::Sortkeys = 1;

use Cwd        qw//;
use FindBin    qw//;
use Hash::Util qw//;

use HTML::Template;
use File::Basename qw//;

use Storable qw//;


#------------------------------------------------------------------------------
# program start
#------------------------------------------------------------------------------
main(@ARGV);
exit;


sub main {
    #--------------------------------------------------------------------------
    # sub          : m a i n
    #--------------------------------------------------------------------------
    # Author       : Christian Dhl
    # Task         : main program
    # Parameter    : none
    # return value : none
    #--------------------------------------------------------------------------
    # 0.0.1  - 2005-02-23 - CD - created
    # 0.0.2  - 2005-02-27 - CD - --nobreak
    #--------------------------------------------------------------------------

    my @args = @_;

    my $ddir    = Cwd::cwd();        # data directory (-> config, sheets)
    my $pdir    = $FindBin::RealBin; # program directory (-> templates)
    my $nobreak = grep { $_ eq '--nobreak' } @args;


    my $config = read_config($ddir);

    create_spip($pdir, $config, $nobreak);
} # sub main


sub read_config {
    #--------------------------------------------------------------------------
    # sub          : r e a d _ c o n f i g
    #--------------------------------------------------------------------------
    # Author       : Christian Dhl
    # Task         : read config.spip
    # Parameter    : 1) directory to config.spip
    # return value : Hashref with config data.
    #--------------------------------------------------------------------------
    # 0.0.1  - 2005-02-23 - CD - created
    # 0.0.3  - 2005-02-28 - CD - Now the presentation can have only one sheet.
    #--------------------------------------------------------------------------

    my ($dir)      = @_;
    my $configfile = $dir . '/config.spip';

    #print "configfile $configfile\n";
    #system("cat $configfile");

    #--------------------------------------------------------------------------
    # Reading the config file with XML::Simple:
    #--------------------------------------------------------------------------
    my $config = XMLin($configfile, forcearray => 0);

    #--------------------------------------------------------------------------
    # Fill optional config data:
    #--------------------------------------------------------------------------
    $config->{css}          = {}             
        unless exists $config->{css};
    $config->{contentsword} = 'Contents'
        unless exists $config->{contentsword};
    $config->{titleword}    = 'Title'
        unless exists $config->{titleword};
    $config->{headpicture}  = undef
        unless exists $config->{headpicture};
    $config->{cssextension} = ''
        unless exists $config->{cssextension};

    Hash::Util::lock_keys(%$config);


    #--------------------------------------------------------------------------
    # Test the fields:
    #--------------------------------------------------------------------------
    die "Missing author in config file $configfile.\n"
        unless exists $config->{author};
    die "Missing datadirectory in config file $configfile.\n"
        unless exists $config->{datadirectory};
    die "Missing date in config file $configfile.\n"
        unless exists $config->{date};
    die "Missing files in config file $configfile.\n"
        unless exists $config->{files};
    die "Missing target in config file $configfile.\n"
        unless exists $config->{target};
    die "Missing title in config file $configfile.\n"
        unless exists $config->{title};
    die "Missing shorttitle in config file $configfile.\n"
        unless exists $config->{shorttitle};
    die "Missing picturedirectory in config file $configfile.\n"
        unless exists $config->{picturedirectory};

    #--------------------------------------------------------------------------
    # Fill css default data:
    #--------------------------------------------------------------------------
    $config->{css}->{pagebackground}      = '#FFF'
        unless exists $config->{css}->{pagebackground};
    $config->{css}->{textcolor}           = '#000'
        unless exists $config->{css}->{textcolor};
    $config->{css}->{textfont}            = 'Verdana'
        unless exists $config->{css}->{textfont};
    $config->{css}->{fontsize}            = '16pt'
        unless exists $config->{css}->{fontsize};
    $config->{css}->{codesize}            = '18pt'
        unless exists $config->{css}->{codesize};
    $config->{css}->{h1color}             = '#00F'
        unless exists $config->{css}->{h1color};
    $config->{css}->{hncolor}             = '#00F'
        unless exists $config->{css}->{hncolor};
    $config->{css}->{linkcolor}           = '#00F'
        unless exists $config->{css}->{linkcolor};
    $config->{css}->{visitedcolor}        = '#009'
        unless exists $config->{css}->{visitedcolor};
    $config->{css}->{hovercolor}         = '#0F0'
        unless exists $config->{css}->{hovercolor};
    $config->{css}->{hoverbackground}     = '#CCC'
        unless exists $config->{css}->{hoverbackground};
    $config->{css}->{activecolor}         = '#F00'
        unless exists $config->{css}->{activecolor};
    $config->{css}->{datecolor}           = '#C0C'
        unless exists $config->{css}->{datecolor};
    $config->{css}->{headcolor}           = '#999'
        unless exists $config->{css}->{headcolor};
    $config->{css}->{headsize}            = '24pt'
        unless exists $config->{css}->{headsize};
    $config->{css}->{headbackground}      = '#000'
        unless exists $config->{css}->{headbackground};
    $config->{css}->{navcolor}            = '#C0C'
        unless exists $config->{css}->{navcolor};
    $config->{css}->{navsize}             = '12pt'
        unless exists $config->{css}->{navsize};
    $config->{css}->{navbackground}       = '#CCC'
        unless exists $config->{css}->{navbackground};
    $config->{css}->{navlinkcolor}        = '#00F'
        unless exists $config->{css}->{navlinkcolor};
    $config->{css}->{navvisitedcolor}     = '#009'
        unless exists $config->{css}->{navvisitedcolor};
    $config->{css}->{navhovercolor}       = '#0F0'
        unless exists $config->{css}->{navhovercolor};
    $config->{css}->{navhoverbackground}  = '#AAA'
        unless exists $config->{css}->{navhoverbackground};
    $config->{css}->{navactivecolor}      = '#F00'
        unless exists $config->{css}->{navactivecolor};
    $config->{css}->{feetcolor}           = '#000'
        unless exists $config->{css}->{feetcolor};
    $config->{css}->{feetsize}            = '12pt'
        unless exists $config->{css}->{feetsize};
    $config->{css}->{feetbackground}      = '#999'
        unless exists $config->{css}->{feetbackground};
    $config->{css}->{feetlinkcolor}       = '#000'
        unless exists $config->{css}->{feetlinkcolor};
    $config->{css}->{feetvisitedcolor}    = '#009'
        unless exists $config->{css}->{feetvisitedcolor};
    $config->{css}->{feethovercolor}      = '#0F0'
        unless exists $config->{css}->{feethovercolor};
    $config->{css}->{feethoverbackground} = '#CCC'
        unless exists $config->{css}->{feethoverbackground};
    $config->{css}->{feetactivecolor}     = '#F00'
        unless exists $config->{css}->{feetactivecolor};

    #--------------------------------------------------------------------------
    # Simplifications and enhancements:
    #--------------------------------------------------------------------------
    $config->{datadirectory}    = $dir . '/' . $config->{datadirectory};
    $config->{target}           = $dir . '/' . $config->{target};
    $config->{files}            = $config->{files}->{file};
    #$config->{picturedirectory} = $dir . '/' . $config->{picturedirectory};

    #--------------------------------------------------------------------------
    # Make files an array if there is only one sheet:
    #--------------------------------------------------------------------------
    if (ref $config->{files} ne 'ARRAY') {
        $config->{files} = [ $config->{files} ];
    }


    return $config;
} # sub read_config


sub create_spip {
    #--------------------------------------------------------------------------
    # sub          : c r e a t e _ s p i p
    #--------------------------------------------------------------------------
    # Author       : Christian Dhl
    # Task         : starts the page creating process
    # Parameter    : 1) directory to the program (for templates)
    #                2) reference to configuration hash
    #                3) Boolean (1=ignore breaks)
    # return value : none
    #--------------------------------------------------------------------------
    # 0.0.1  - 2005-02-23 - CD - created
    # 0.0.2  - 2005-02-27 - CD - --nobreak
    #--------------------------------------------------------------------------

    my ($dir, $config, $nobreak) = @_;
 
    #--------------------------------------------------------------------------
    # create target directory:
    #--------------------------------------------------------------------------
    mkdir($config->{target}) unless -d $config->{target};

    #--------------------------------------------------------------------------
    # css creation from template:
    #--------------------------------------------------------------------------
    my $cssfile = $config->{target} . "/spip.css";
    create_css($dir, $config, $cssfile);

    #--------------------------------------------------------------------------
    # title creation from template:
    #--------------------------------------------------------------------------
    create_title($dir, $config, $config->{target} . "/index.html", $cssfile);

    #--------------------------------------------------------------------------
    # work on sheets:
    #--------------------------------------------------------------------------
    my @pages = work_on_sheets($config, $cssfile);

    #--------------------------------------------------------------------------
    # split at the breaks:
    #--------------------------------------------------------------------------
    my %chapter;
    my @totalpages = split_the_breaks(\@pages, \%chapter, $nobreak);
    undef @pages;
        
    #--------------------------------------------------------------------------
    # create the navigation data and the contents:
    #--------------------------------------------------------------------------
    my $content = create_navigation($config, \@totalpages);

    #--------------------------------------------------------------------------
    # create the contents file:
    #--------------------------------------------------------------------------
    my $contentsfile = $config->{target} . "/contents.html";
    create_contents($dir, $config, $contentsfile, $content, $cssfile, \%chapter);

    #--------------------------------------------------------------------------
    # create html pages:
    #--------------------------------------------------------------------------
    for my $totalpage (@totalpages) {
        for my $page (@$totalpage) {
            print "create ", $page->{file}, " ...\n";
            create_sheet($dir, $config, $page);
        }
    }
} # sub create_spip


sub create_css {
    #--------------------------------------------------------------------------
    # sub          : c r e a t e _ c s s
    #--------------------------------------------------------------------------
    # Author       : Christian Dhl
    # Task         : creates the css file
    # Parameter    : 1) directory to the program (for templates)
    #                2) reference to configuration hash
    #                3) name of the css file
    # return value : none
    #--------------------------------------------------------------------------
    # 0.0.1  - 2005-02-23 - CD - created
    #--------------------------------------------------------------------------

    my ($dir, $config, $cssfile) = @_;

    #--------------------------------------------------------------------------
    # Create template object:
    #--------------------------------------------------------------------------
    my $template = HTML::Template->new(
        filename => $dir . '/templates/spip_css.tmpl'
    );

    #--------------------------------------------------------------------------
    # Fill in the parameters:
    #--------------------------------------------------------------------------
    $template->param(PAGEBACKGROUND      => $config->{css}->{pagebackground}     );
    $template->param(TEXTCOLOR           => $config->{css}->{textcolor}          );
    $template->param(TEXTFONT            => $config->{css}->{textfont}           );
    $template->param(FONTSIZE            => $config->{css}->{fontsize}           );
    $template->param(CODESIZE            => $config->{css}->{codesize}           );
    $template->param(H1COLOR             => $config->{css}->{h1color}            );
    $template->param(HNCOLOR             => $config->{css}->{hncolor}            );
    $template->param(LINKCOLOR           => $config->{css}->{linkcolor}          );
    $template->param(VISITEDCOLOR        => $config->{css}->{visitedcolor}       );
    $template->param(HOVERCOLOR          => $config->{css}->{hovercolor}         );
    $template->param(HOVERBACKGROUND     => $config->{css}->{hoverbackground}    );
    $template->param(ACTIVECOLOR         => $config->{css}->{activecolor}        );
    $template->param(DATECOLOR           => $config->{css}->{datecolor}          );
    $template->param(FEETCOLOR           => $config->{css}->{feetcolor}          );
    $template->param(FEETSIZE            => $config->{css}->{feetsize}           );
    $template->param(FEETBACKGROUND      => $config->{css}->{feetbackground}     );
    $template->param(NAVCOLOR            => $config->{css}->{navcolor}           );
    $template->param(NAVSIZE             => $config->{css}->{navsize}            );
    $template->param(NAVBACKGROUND       => $config->{css}->{navbackground}      );
    $template->param(HEADCOLOR           => $config->{css}->{headcolor}          );
    $template->param(HEADSIZE            => $config->{css}->{headsize}           );
    $template->param(HEADBACKGROUND      => $config->{css}->{headbackground}     );
    $template->param(NAVLINKCOLOR        => $config->{css}->{navlinkcolor}       );
    $template->param(NAVVISITEDCOLOR     => $config->{css}->{navvisitedcolor}    );
    $template->param(NAVHOVERCOLOR       => $config->{css}->{navhovercolor}      );
    $template->param(NAVHOVERBACKGROUND  => $config->{css}->{navhoverbackground} );
    $template->param(NAVACTIVECOLOR      => $config->{css}->{navactivecolor}     );
    $template->param(FEETLINKCOLOR       => $config->{css}->{feetlinkcolor}      );
    $template->param(FEETVISITEDCOLOR    => $config->{css}->{feetvisitedcolor}   );
    $template->param(FEETHOVERCOLOR      => $config->{css}->{feethovercolor}     );
    $template->param(FEETHOVERBACKGROUND => $config->{css}->{feethoverbackground});
    $template->param(FEETACTIVECOLOR     => $config->{css}->{feetactivecolor}    );

    $template->param(CSSEXTENSION        => $config->{cssextension}              );

    #--------------------------------------------------------------------------
    # Write out the file:
    #--------------------------------------------------------------------------
    {
        print "create ", File::Basename::basename($cssfile), " ...\n";
        my $css = new IO::File '>' . $cssfile
            or die "Can't open '$cssfile': $!";
        print $css $template->output();
    }
} # sub create_css


sub create_title {
    #--------------------------------------------------------------------------
    # sub          : c r e a t e _ t i t l e
    #--------------------------------------------------------------------------
    # Author       : Christian Dhl
    # Task         : creates the title file (index.html)
    # Parameter    : 1) directory to the program (for templates)
    #                2) reference to configuration hash
    #                3) name of the title file
    #                4) name of the css file
    # return value : none
    #--------------------------------------------------------------------------
    # 0.0.1  - 2005-02-23 - CD - created
    #--------------------------------------------------------------------------

    my ($dir, $config, $titlefile, $cssfile) = @_;

    #--------------------------------------------------------------------------
    # Create template object:
    #--------------------------------------------------------------------------
    my $template = HTML::Template->new(
        filename => $dir . '/templates/spip_title.tmpl'
    );

    my %param = (
        TITLE        => $config->{title},
        CSSFILE      => File::Basename::basename($cssfile),
        AUTHOR       => $config->{author},
       #DATE         => $config->{date},
        PAGEBODY     => <<ENDE,
        <div class="center">
            <p>&nbsp;</p>
            <p style="font-size:48pt;">$config->{shorttitle}</p>
            <p>&nbsp;</p>
            <p style="font-size:36pt;">$config->{title}</p>
            <p>&nbsp;</p>
            <p style="font-size:24pt;">$config->{author}</p>
            <p>&nbsp;</p>
            <p style="font-size:18pt;">$config->{date}</p>
            <p>&nbsp;</p>
        </div>
ENDE
    );
    $param{HEADPICTURE} = '../' . $config->{picturedirectory}
                          . '/' . $config->{headpicture}
        if exists  $config->{headpicture} and
           defined $config->{headpicture};


    #--------------------------------------------------------------------------
    # Fill in the parameters:
    #--------------------------------------------------------------------------
    for my $p (keys %param) {
        $template->param($p => $param{$p});
    }

    #--------------------------------------------------------------------------
    # Write out the file:
    #--------------------------------------------------------------------------
    {
        print "create ", File::Basename::basename($titlefile), " ...\n";
        my $title = new IO::File '>' . $titlefile
            or die "Can't open '$titlefile': $!";
        print $title $template->output();
    }
} # sub create_title


sub work_on_sheets {
    #--------------------------------------------------------------------------
    # sub          : w o r k _ o n _ s h e e t s
    #--------------------------------------------------------------------------
    # Author       : Christian Dhl
    # Task         : Reading all the sheets and create @pages array.
    # Parameter    : 1) reference to configuration hash
    #                2) Name of the css file
    # return value : @pages
    #--------------------------------------------------------------------------
    # 0.0.1  - 2005-02-27 - CD - created
    #--------------------------------------------------------------------------

    my ($config, $cssfile) = @_;

    my @pages;
    
    for my $file (@{ $config->{files} }) {
        my $page = {};
        $page->{sheet} = read_sheet($config->{datadirectory} . "/$file.spip");
        $page->{file}  = $file;
        $page->{param} = {
            TITLE        => $config->{shorttitle} . ' - '
                            . $page->{sheet}->{chapter},
            PAGETITLE    => $page->{sheet}->{subchapter},
            CSSFILE      => File::Basename::basename($cssfile),
            AUTHOR       => $config->{author},
            PAGEBODY     => $page->{sheet}->{data},
            DATE         => $config->{date},
            CONTENTS     => 'contents.html',
            CONTENTSWORD => $config->{contentsword},
            HEADTITLE    => $config->{title},
        };
        $page->{param}->{PICTURE} = '../' . $config->{picturedirectory}
                                    . '/' . $page->{sheet}->{picture}
            if exists $page->{sheet}->{picture};
        $page->{param}->{HEADPICTURE} = '../' . $config->{picturedirectory}
                                        . '/' . $config->{headpicture}
            if exists  $config->{headpicture} and
               defined $config->{headpicture};

        $page->{sheet}->{data} = ''; # we don't need that any more

        push @pages, $page;
    }

    return @pages;
} # sub work_on_sheets


sub read_sheet {
    #--------------------------------------------------------------------------
    # sub          : r e a d _ s h e e t
    #--------------------------------------------------------------------------
    # Author       : Christian Dhl
    # Task         : read sheet file
    # Parameter    : 1) filename (absolute)
    # return value : Hashref with sheet data.
    #--------------------------------------------------------------------------
    # 0.0.1  - 2005-02-23 - CD - created
    #--------------------------------------------------------------------------

    my ($file) = @_;

    print "reading file ", File::Basename::basename($file), " ...\n";
    
    my $sheet = XMLin($file);
    Hash::Util::lock_keys(%$sheet);
    my $in = new IO::File '<' . $file or die "Can't open '$file': $!";
    my $filecontent;
    { local $/; $filecontent = <$in>;}
    ($sheet->{data}) = $filecontent =~ m~<data>\s*\n(.*)\n\s*</data>~s;

    return $sheet;

} # sub read_sheet


sub split_the_breaks {
    #--------------------------------------------------------------------------
    # sub          : s p l i t _ t h e _ b r e a k s
    #--------------------------------------------------------------------------
    # Author       : Christian Dhl
    # Task         : Split the breaks and create all pages. Create @totalpages
    #                array.
    # Parameter    : 1) reference to the pages array
    #                2) reference to the chapter hash (to be filled)
    #                3) Boolean (1=ignore breaks)
    # return value : @totalpages
    #--------------------------------------------------------------------------
    # 0.0.1  - 2005-02-27 - CD - created
    #--------------------------------------------------------------------------

    my ($pages, $chapter, $nobreak) = @_;

    my @totalpages;
    my $nr = 0;
    for my $page (@$pages) {
        my $subpages = [];

        $page->{file} .= '_01';

        $chapter->{$page->{sheet}->{chapter}}->{content}->
            {$page->{sheet}->{subchapter}}->{content}
            = $page->{file} . '.html';
        push @{ $chapter->{$page->{sheet}->{chapter}}->{number} }, $nr;
        push @{ $chapter->{$page->{sheet}->{chapter}}->{content}->
                {$page->{sheet}->{subchapter}}->{number}
              }, $nr;
        ++$nr;

        if ($nobreak) {
            $page->{param}->{PAGEBODY}  =~ s~(?:^\s*)?{{{BREAK}}}(?:\s*$)~~mg;
        }

        while ($page->{param}->{PAGEBODY} =~ m~{{{BREAK}}}~) {
            my $clone = Storable::dclone($page);
            my $nr    = sprintf "%02d", $page->{file} =~ m~_(\d\d)$~;
            my $nnr   = sprintf "%02d", $nr+1;
            $clone->{file} =~ s~_$nr$~_$nnr~;

            $page->{param}->{PAGEBODY}  =~ s~{{{BREAK}}}.*~~s;
            $page->{param}->{PAGEBODY}  = $page->{param}->{PAGEBODY}
                                          . '<p class="smaller"><a href="'
                                          . $clone->{file}
                                          . '.html">continue</a></p>';
            $clone->{param}->{PAGEBODY} =~ s~^.*{{{BREAK}}}.*$~~m;
            push @$subpages, $page;
            $page = $clone;
            undef $clone;
        }
        push @$subpages, $page;
        push @totalpages, $subpages;
    }

    return @totalpages;
} # sub split_the_breaks


sub create_navigation {
    #--------------------------------------------------------------------------
    # sub          : c r e a t e _ n a v i g a t i o n
    #--------------------------------------------------------------------------
    # Author       : Christian Dhl
    # Task         : Creates the navigation in each page and collects the
    #                contents page data.
    # Parameter    : 1) reference to configuration hash
    #                2) reference to the totalpages array
    # return value : $content ( Contents page data)
    #--------------------------------------------------------------------------
    # 0.0.1  - 2005-02-27 - CD - created
    #--------------------------------------------------------------------------

    my ($config, $totalpages) = @_;

    my $content = qq(<ul>\n<li><a href="index.html">) . $config->{titleword}
                  . qq(</a><br /><br /></li>\n);
    for my $tpi (0..$#$totalpages) {
        $content .= '<li><a href="' . $totalpages->[$tpi]->[0]->{file} . '.html'
                    . '">' . $totalpages->[$tpi]->[0]->{param}->{PAGETITLE}
                    . "</a></li>\n";

        for my $pi (0..$#{$totalpages->[$tpi]}) {
            my $page = $totalpages->[$tpi]->[$pi];
            #print "creating navigation ", $page->{file}, " ...\n";

            $page->{param}->{PREVIOUS} =
                $totalpages->[$tpi]->[$pi-1]->{file} . '.html'
                if $pi > 0;
            $page->{param}->{NEXT} =
                $totalpages->[$tpi]->[$pi+1]->{file} . '.html'
                if $pi < $#{$totalpages->[$tpi]};
            $page->{param}->{CHAPPREV} =
                $totalpages->[$tpi-1]->[0]->{file} . '.html'
                if $tpi > 0;
            $page->{param}->{CHAPNEXT} =
                $totalpages->[$tpi+1]->[0]->{file} . '.html'
                if $tpi < $#$totalpages;
            $page->{param}->{LAST} =
                $totalpages->[$tpi]->[$#{$totalpages->[$tpi]}]->{file} . '.html'
                if $#{$totalpages->[$tpi]} > 0 and
                   $pi != $#{$totalpages->[$tpi]};
        }
    }
    $content .= "</ul>\n";

    return $content;
} # sub create_navigation


sub create_contents {
    #--------------------------------------------------------------------------
    # sub          : c r e a t e _ c o n t e n t s
    #--------------------------------------------------------------------------
    # Author       : Christian Dhl
    # Task         : creates the contents file
    # Parameter    : 1) directory to the program (for templates)
    #                2) reference to configuration hash
    #                3) name of the contents file
    #                4) the contents entries
    #                5) name of the css file
    #                6) Hash structure with chapters and subchapters and
    #                   filenames.
    # return value : none
    #--------------------------------------------------------------------------
    # 0.0.1  - 2005-02-23 - CD - created
    #--------------------------------------------------------------------------

    my ($dir, $config, $contentsfile, $content, $cssfile, $chapter) = @_;

    my $chapcont = qq(<ol class="smaller">\n)
                   . q(    <li><a href="index.html">) . $config->{titleword}
                   . qq(</a><br /><br /></li>\n);

    my $linecount = 0;
    my $data;
    for my $chap (sort {$chapter->{$a}->{number}->[0] <=>
                        $chapter->{$b}->{number}->[0]}
                  keys %$chapter)
    {
        $chapcont .= '    <li>' . $chap . "\n"
                     . "    <ol>\n";
        ++$linecount;
        for my $subchap (sort {$chapter->{$chap}->{content}->{$a}->{number}->[0] <=>
                               $chapter->{$chap}->{content}->{$b}->{number}->[0]}
                         keys %{$chapter->{$chap}->{content}})
        {
            $chapcont .= qq(        <li><a href=")
                         . $chapter->{$chap}->{content}->{$subchap}->{content}
                         . qq(">$subchap</a></li>\n);
            ++$linecount;
        }
        $chapcont .= "    </ol></li>\n";
    }
    $chapcont .= qq(</ol>\n);

    #--------------------------------------------------------------------------
    # Split content if it is long:
    #--------------------------------------------------------------------------
    if ($linecount > 14) {
        my @cc = split /\n/, $chapcont;
        my $i = int($#cc / 2);
        while (not $cc[$i] =~ m~^    <li>~) {
            ++$i;
            last if $i > $#cc;
        }
        if ($i < $#cc) {
            my $cpc1 = join("\n", @cc[0..$i-1]) . "\n</ol>\n";
            my $c = () = grep {m~^    <li>~} @cc[0..$i-1];
            ++$c;
            my $cpc2 = qq(<ol start="$c" class="smaller">\n)
                       . join("\n", @cc[$i..$#cc]) . "\n";
            $data = qq(<div class="l5050">\n) . $cpc1 . qq(</div>\n) .
                    qq(<div class="r5050">\n) . $cpc2 . qq(</div>\n);
        }
        else {
            $data = qq(<div>\n) . $chapcont  . qq(</div>\n);
        }
    }
    else {
        #----------------------------------------------------------------------
        # Fill data:
        #----------------------------------------------------------------------
        $data = qq(<div>\n) . $chapcont  . qq(</div>\n);
    }

    #--------------------------------------------------------------------------
    # Create template object:
    #--------------------------------------------------------------------------
    my $template = HTML::Template->new(
        filename => $dir . '/templates/spip_contents.tmpl'
    );

    my %param = (
        TITLE        => $config->{title} . ' - '
                        . $config->{contentsword},
        PAGETITLE    => $config->{contentsword},
        CSSFILE      => File::Basename::basename($cssfile),
        AUTHOR       => $config->{author},
        PAGEBODY     => $data,
        DATE         => $config->{date},
        HEADTITLE    => $config->{title},
    );
    $param{HEADPICTURE} = '../' . $config->{picturedirectory}
                          . '/' . $config->{headpicture}
        if exists  $config->{headpicture} and
           defined $config->{headpicture};

    #--------------------------------------------------------------------------
    # Fill in the parameters:
    #--------------------------------------------------------------------------
    for my $p (keys %param) {
        $template->param($p => $param{$p});
    }

    #--------------------------------------------------------------------------
    # Write out the file:
    #--------------------------------------------------------------------------
    {
        print "create ", File::Basename::basename($contentsfile), " ...\n";
        my $contents = new IO::File '>' . $contentsfile
            or die "Can't open '$contentsfile': $!";
        print $contents $template->output();
    }
} # sub create_contents


sub create_sheet {
    #--------------------------------------------------------------------------
    # sub          : c r e a t e _ s h e e t
    #--------------------------------------------------------------------------
    # Author       : Christian Dhl
    # Task         : creates the css file
    # Parameter    : 1) directory to the program (for templates)
    #                2) reference to configuration hash
    #                3) hashref with page data
    # return value : none
    #--------------------------------------------------------------------------
    # 0.0.1  - 2005-02-23 - CD - created
    #--------------------------------------------------------------------------

    my ($dir, $config, $page) = @_;

    #--------------------------------------------------------------------------
    # insert code {{{COLORCODE:file.pl}}}
    #--------------------------------------------------------------------------
    $page->{param}->{PAGEBODY} =~
        s~{{{COLORCODE:([-a-zA-Z0-9_.]+)}}}~color_read($config->{datadirectory} . "/$1")~eg;

    #--------------------------------------------------------------------------
    # open the html template
    #--------------------------------------------------------------------------
    my $template = HTML::Template->new(
        filename => $dir . '/templates/spip_sheet.tmpl'
    );

    #--------------------------------------------------------------------------
    # fill in some parameters
    #--------------------------------------------------------------------------
    for my $param (keys %{$page->{param} }) {
        $template->param($param => $page->{param}->{$param});
    }

    #--------------------------------------------------------------------------
    # print the template output
    #--------------------------------------------------------------------------
    {
        my $htmlfile = $config->{target} . '/' . $page->{file} . '.html';
        my $html = new IO::File '>' . $htmlfile
            or die "Can't open '$htmlfile': $!";
        print $html $template->output();
    }
} # sub create_sheet


sub color_read {
    #--------------------------------------------------------------------------
    # sub          : c o l o r _ r e a d
    #--------------------------------------------------------------------------
    # Author       : Christian Dhl
    # Task         : Reads colorized source code and prepares it for html.
    # Parameter    : 1) file to read
    # return value : String with code.
    #--------------------------------------------------------------------------
    # 0.0.1  - 2005-02-24 - CD - created
    #--------------------------------------------------------------------------

    my ($file) = @_;

    #my $ret = '<tt><span class="black">';
    my $ret = '<tt>';

    my $spanopen = 0;
    open (F, $file) or die "Can't read file '$file': $!";
    while (<F>) {
        chomp;
        s~ ~&nbsp;~g;

        while (m~({{{([A-Z]+)}}})~g) {
            my ($full, $inner) = ($1, $2);
            if ($inner eq 'OFF') {
                die "codeerror in file '$file' line $." if $spanopen == 0;
                s~\Q$full\E~</strong></span>~;
                $spanopen = 0;
            }
            else {
                my $sinner = lc $inner;
                if (exists {qw/black 1 gray 1 silver 1 white 1 purple 1
                               fuchsia 1 maroon 1 red 1 olive 1 yellow 1
                               green 1 lime 1 navy 1 blue 1 teal 1 aqua 1
                               orange 1/}->{$sinner})
                {
                    if ($spanopen == 1) {
                        s~\Q$full\E~</strong></span><span class="$sinner"><strong>~;
                    }
                    else {
                        s~\Q$full\E~<span class="$sinner"><strong>~;
                        $spanopen = 1;
                    }
                }
            }
        }
        $ret .= $_ . "<br />\n";
    }
    $ret .= '</tt>';

    return $ret;
} # sub color_read


__END__

=head1 NAME

SPIP - spip.pl - Simple Presentations in Perl

=head1 PARAMETER

=over

=item C<--nobreak>

Ignore the breaks.

=back

=head1 DOCUMENTATION

See example presentation in the directory 'documentation' (html/index.html)

=head1 COPYRIGHT

Copyright 2005 Christian Duehl. All rights reserved.

This program is free software. You can redistribute it and/or
modify it under the same terms as perl itself.


=cut

